/*
 * splashwidget.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef SPLASHWIDGET_H
#define SPLASHWIDGET_H

#include <khtml_part.h>

namespace Barry
{
	class SplashWidget : public KHTMLPart
	{
		Q_OBJECT
		public:
			SplashWidget( QWidget *parent, const char *name = 0 );

		private:
			bool loadStyleSheet();
			bool showSplashScreen();
			void showErrorMessage( const QString &msg );

			QString langLookup( const QString &fileName ) const;
	};
};

#endif // SPLASHWIDGET_H
// vim:ts=4:sw=4:noet:list
