/*
 * infowidget.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef INFOWIDGET_H
#define INFOWIDGET_H

#include "port.h"

#include <qwidget.h>

class KTextBrowser;
class KURLLabel;
class QLabel;

namespace Barry
{
	class InfoWidget : public QWidget
	{
		Q_OBJECT
		public:
			InfoWidget( QWidget *parent );

		public slots:
			void setPort( const Port &port );

		private slots:
			void openURL( const QString &url );

		private:
			QLabel *m_packageName;
			QLabel *m_version;
			QLabel *lCategories;
			QLabel *m_categories;
			KURLLabel *m_maintainer;
			QLabel *m_comment;
			KTextBrowser *m_description;
			Port m_port;
	};
};

#endif // INFOWIDGET_H
// vim:ts=4:sw=4:noet:list
