/*
 * updatedpackagesdialog.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef UPDATEDPACKAGESDIALOG_H
#define UPDATEDPACKAGESDIALOG_H

#include "actionprogressdialog.h"

namespace Barry
{
	class Port;
	class PortItem;

	class UpdatedPackagesProgressDialog : public ActionProgressDialog
	{
		Q_OBJECT
		public:
			UpdatedPackagesProgressDialog( QWidget *parent, const char *name = 0 );

		protected:
			virtual void processPort( const Port &port );
			virtual void setupItem( PortItem *item );
	};
};

#endif // UPDATEDPACKAGESDIALOG_H
// vim:ts=4:sw=4:noet:list
