/*
 * searchwidget.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef SEARCHWIDGET_H
#define SEARCHWIDGET_H

#include <qwidget.h>

class KLineEdit;
class QToolButton;

namespace Barry
{
	class SearchWidget : public QWidget
	{
		Q_OBJECT
		public:
			SearchWidget( QWidget *parent );
			
		signals:
			void searchTerm( const QString &term );

		public slots:
			void addEntry( const QString &entry );

		private slots:
			void contentsChanged( const QString &contents );
			void gotSearchTerm( const QString &term );
			void buttonPressed();

		private:
			KLineEdit *m_editField;
			QToolButton *m_toolButton;
	};
};

#endif // SEARCHWIDGET_H
// vim:ts=4:sw=4:noet:list
