/*
 * systemtraywin.h
 *
 * Copyright (c) 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef SYSTEMTRAYWIN_H
#define SYSTEMTRAYWIN_H

#include <ksystemtray.h>

class KMainWindow;

namespace Barry
{
	class SystemTrayWin : public KSystemTray
	{
		Q_OBJECT
		public:
			SystemTrayWin( KMainWindow *parent = 0, const char *name = 0 );
	};
};

#endif // SYSTEMTRAYWIN_H
// vim:ts=4:sw=4:noet
