/*
 * dependlistview.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef DEPENDLISTVIEW_H
#define DEPENDLISTVIEW_H

#include "port.h"

#include <klistview.h>

namespace Barry
{
	class DependancyList : public KListView
	{
		Q_OBJECT
		public:
			enum Type { BuildDeps, RunDeps };
			DependancyList( QWidget *parent, Type type );

		public slots:
			void setPort( const Port &port );

		private slots:
			void itemSelected( QListViewItem *item );

		private:
			Port m_port;
			Type m_type;
	};
};

#endif // DEPENDLISTVIEW_H
// vim:ts=4:sw=4:noet:list
