<!DOCTYPE kpartgui>
<kpartgui name="Barry" version="9">
<MenuBar>
	<Menu name="file">
		<Action name="database_info"/>
	</Menu>
	<Menu name="edit">
<!--
		<Separator/>
		<Action name="port_install"/>
		<Action name="port_upgrade"/>
		<Action name="port_deinstall"/>
-->
		<Separator/>
		<Action name="view_webcvs"/>
	</Menu>
	<Menu name="go">
		<text>&amp;Go</text>
		<Action name="back"/>
		<Action name="forward"/>
		<Separator/>
	</Menu>
	<Menu name="tools">
		<Action name="find_unused_pkgs"/>
		<Action name="find_updated_pkgs"/>
		<Action name="find_orphaned_files"/>
	</Menu>
</MenuBar>
<ToolBar>
	<Action name="back"/>
	<Action name="forward"/>
	<Action name="options_configure"/>
</ToolBar>
<!--
<ToolBar name="portActions"><text>Port actions</text>
	<Action name="port_install"/>
	<Action name="port_upgrade"/>
	<Action name="port_deinstall"/>
</ToolBar>
-->
</kpartgui>
