/*
 * main.cc
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "application.h"

#include <kaboutdata.h>
#include <kcmdlineargs.h>

static const char * const appName = "barry";
static const char * const programName = "Barry";
static const char * const version = VERSION;
static const char * const description = I18N_NOOP( "Port manager for BSD-based systems" );
static KCmdLineOptions options[] =
{
	{ "rebuild-cache", I18N_NOOP( "Enforce rebuilding the cache" ), 0},
	{ "s", 0, 0 },
	{ "search <term>", I18N_NOOP( "Search for the specified term in the " \
	                             "port database" ), 0 },
	{ 0, 0, 0 }
};

int main( int argc, char **argv )
{
	KAboutData aboutData( appName, programName, version, description,
	                      KAboutData::License_BSD );
	aboutData.addAuthor( "Frerich Raabe", I18N_NOOP( "Original author" ),
	                     "raabe@kde.org" );
	aboutData.addCredit( "Dirk Doerflinger", I18N_NOOP( "Had the original idea." ),
	                     "ddoerflinger@web.de" );
	aboutData.addCredit( "Akinori Musha", I18N_NOOP( "Granted me lots of "
	                     "insight in how the BSD port & package systems work." ),
	                     "knu@FreeBSD.org" );
	aboutData.addCredit( "Lauri Watts", I18N_NOOP( "Stresstesting" ),
	                     "lauri@kde.org" );
	aboutData.addCredit( "Andy Fawcett", I18N_NOOP( "Generously provided the"
	                     " webspace for the homepage." ),
	                     "andy@athame.co.uk" );
	aboutData.setTranslator(
		I18N_NOOP( "_: NAME OF TRANSLATORS\nYour names" ),
		I18N_NOOP( "_: EMAIL OF TRANSLATORS\nYour emails" )
	);

	KLocale::setMainCatalogue( "barry" );
	KCmdLineArgs::init( argc, argv, &aboutData );
	KCmdLineArgs::addCmdLineOptions( options );
	KUniqueApplication::addCmdLineOptions();

	Barry::Application app;

	return app.exec();
}
// vim:ts=4:sw=4:noet:list
