/*
 * systemtraywin.cc
 *
 * Copyright (c) 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#include "systemtraywin.h"

#include <kaction.h>
#include <kapplication.h>
#include <kmainwindow.h>
#include <kpopupmenu.h>

using namespace Barry;

SystemTrayWin::SystemTrayWin( KMainWindow *parent, const char *name )
	: KSystemTray( parent, name )
{
	setPixmap( kapp->miniIcon() );

	KPopupMenu *menu = contextMenu();

	KAction *findAction = parent->actionCollection()->action( "edit_find" );
	if ( findAction )
		findAction->plug( menu );
}

#include "systemtraywin.moc"
// vim:ts=4:sw=4:noet
