/*
 * configdialog.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef CONFIGDIALOG_H
#define CONFIGDIALOG_H

#include <kdialogbase.h>

class QSpinBox;

class KComboBox;
class KListView;
class KURLRequester;

namespace Barry
{
	class ConfigDialog : public KDialogBase
	{
		Q_OBJECT
		public:
			ConfigDialog( QWidget *parent );

		protected slots:
			virtual void slotOk();

		private slots:
			void addFile();
			void removeFile();
			void selectionChanged( QListViewItem *item );

		private:
			void loadConfig();
			void saveConfig();
			void setupGeneralPage();
			void setupOrphanedFilesPage();

			KURLRequester *m_supfile;
			KURLRequester *m_portDir;
			KURLRequester *m_pkgDir;
			KComboBox *m_cvswebMirror;
			QSpinBox *m_historyEntries;

			KListView *m_ignoredFiles;
			QPushButton *m_addFile;
			QPushButton *m_removeFile;
	};
};

#endif // CONFIGDIALOG_H
// vim:ts=4:sw=4:noet:list
