/*
 * application.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef APPLICATION_H
#define APPLICATION_H

#include <kuniqueapplication.h>

namespace Barry
{
	class MainWindow;
	
	class Application : public KUniqueApplication
	{
		Q_OBJECT
		public:
			Application();

			virtual int newInstance();

		public slots:
			void rebuildCache();
	};
};

#endif // APPLICATION_H
// vim:ts=4:sw=4:noet:list
