/*
 * databaseinfodialog.h
 *
 * Copyright (c) 2002, 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef DATABASEINFODIALOG_H
#define DATABASEINFODIALOG_H

#include <kdialogbase.h>

class QLabel;

namespace Barry
{
	class DatabaseInfoDialog : public KDialogBase
	{
		Q_OBJECT
		public:
			DatabaseInfoDialog( QWidget *parent );

		protected slots:
			virtual void slotUser1();
			virtual void slotUser2();

		private:
			QString cacheDirName() const;
			unsigned int cachedItems() const;
			unsigned long cacheSize() const;
			void showData();

			QLabel *m_portTimeStamp;
			QLabel *m_packageTimeStamp;
			QLabel *m_cachedItems;
			QLabel *m_cacheSize;
	};
};

#endif // DATABASEINFODIALOG_H
// vim:ts=4:sw=4:noet:list
